#include "hope.h"

void init(void)
{
  glClearColor(0.0, 0.0, 0.0, 0.0);
  glShadeModel(GL_FLAT);
}

void camera(void)
{
  glRotatef(p1.roll, 0.0, 0.0, 1.0);
  glRotatef(p1.pitch, 0.0, 1.0, 0.0);
  glRotatef(p1.yaw, 1.0, 0.0, 0.0);
  glTranslatef(p1.x, p1.y, p1.z);
}

void camera_swiftless (void) {
	glRotatef(p1.yaw,1.0,0.0,0.0);  //rotate our camera on teh x-axis (left and right)
	glRotatef(p1.pitch,0.0,1.0,0.0);  //rotate our camera on the y-axis (up and down)
	glTranslated(-p1.x,-p1.y,-p1.z); //translate the screen to the position of our camera
}

void drawGeometry(void)
{
  int x,y,z;
  for (x = 0; x < d; x++)
  {
    for (y = 0; y < d; y++)
    {
      for (z = 0; z < d; z++)
      {
	if (v[x][y][z])
	{
	  glPushMatrix();
	  glTranslatef(x - 10, y - 10, z - 10);
	  glutSolidCube(1.0);
	  glPopMatrix();
	}
      }
    }
  }  
}

// copied from swiftless tutorial
void enable(void)
{
  glEnable (GL_DEPTH_TEST); //enable the depth testing
  glEnable (GL_LIGHTING); //enable the lighting
  glEnable (GL_LIGHT0); //enable LIGHT0, our Diffuse Light
  glShadeModel (GL_SMOOTH); //set the shader to smooth shader
}

void display(void)
{
  //glClear(GL_COLOR_BUFFER_BIT);
  //glColor3f(1.0, 1.0, 1.0);
  glClearColor(0,0,0,1);
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); //clear the color buffer and the depth buffer
  glLoadIdentity();
  camera_swiftless();
  enable();
  //gluLookAt (eyeX, eyeY, eyeZ, centerX, centerY, centerZ, upX, upY, upZ);
  //glScalef(1.0, 1.0, 1.0);
  drawGeometry();
  glFlush();
}

void reshape (int w, int h)
{
  glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity ();
  glFrustum (-1.0, 1.0, -1.0, 1.0, 1.5, 20.0);
  glMatrixMode (GL_MODELVIEW);
}

void keyboard(unsigned char key, int x, int y)
{
  switch (key)
  {
    case 'w': p1.z += 0.1; break;
    case 's': p1.z -= 0.1; break;
    case 'a': p1.x += 0.1; break;
    case 'd': p1.x -= 0.1; break;
    case ' ': p1.y -= 0.1; break;
    case 'c': p1.y += 0.1; break;
    case 'o': p1.pitch += 1; if (p1.pitch > 360) p1.pitch -= 360; break;
    case 'l': p1.pitch -= 1; if (p1.pitch < -360) p1.pitch += 360; break;
    case 'r': p1.yaw += 1; if (p1.yaw > 360) p1.yaw -= 360; break;
    case 'u': p1.yaw -= 1; if (p1.yaw < -360) p1.yaw += 360; break;
    case 'k': p1.roll += 1; if (p1.roll > 360) p1.roll -= 360; break;
    case ';': p1.roll -= 1; if (p1.roll < -360) p1.roll += 360; break;
    case 27:
      exit(EXIT_SUCCESS);
      break;
  }
  glutPostRedisplay();
}

void keyboard_swiftless (unsigned char key, int x, int y) {
	if (key=='q')
	{
	p1.yaw += 1;
	if (p1.yaw >360) p1.yaw -= 360;
	}

	if (key=='z')
	{
	p1.yaw -= 1;
	if (p1.yaw < -360) p1.yaw += 360;
	}

	if (key=='w')
	{
	float yawRadians, pitchRadians;
	pitchRadians = (p1.pitch / 180 * M_PI);
	yawRadians = (p1.yaw / 180 * M_PI); 
	p1.x += sin(pitchRadians);
	p1.z -= cos(pitchRadians);
	p1.y -= sin(yawRadians);
	}

	if (key=='s')
	{
	float yawRadians, pitchRadians;
	pitchRadians = (p1.pitch / 180 * M_PI);
	yawRadians = (p1.yaw / 180 * M_PI); 
	p1.x -= sin(pitchRadians);
	p1.z += cos(pitchRadians);
	p1.y += sin(yawRadians);
	}

	if (key=='d')
	{
	p1.pitch += 1;
	if (p1.pitch >360) p1.pitch -= 360;
	}

	if (key=='a')
	{
	p1.pitch -= 1;
	if (p1.pitch < -360)p1.pitch += 360;
	}
	if (key==27)
	{
	exit(0);
	}
	
	// debugging keystroke, TODO: disable in release versions
	if (key == 'p')
	{
	  printf("X = %f\nY = %f\nZ = %f\nPitch = %f\nYaw = %f\nRoll = %f\n",
		 p1.x, p1.y, p1.z, p1.pitch, p1.yaw, p1.roll);
	}
}

int main (int argc, char * argv[])
{
  int x,y,z;
  
  bool i[d][d];
  fractInit(d, i);
  
  for (y=0;y<d;y++)
  {
    for (x=0;x<d;x++)
    {
      for (z=0;z<d;z++)
      {
	v[x][y][z] = (i[x][y] && i[z][y]);
      }
    }
  }
  
  glutInit(&argc, argv);
  glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB);
  glutInitWindowSize (500, 500); 
  glutInitWindowPosition (100, 100);
  glutCreateWindow (argv[0]);
  init ();
  glutDisplayFunc(display);
  glutIdleFunc(display);
  glutReshapeFunc(reshape);
  glutKeyboardFunc(keyboard_swiftless);
  glutMainLoop();
  
  return(EXIT_SUCCESS);
}

void fractInit(int size, bool i[][size])
{
  int x, y;
  for (x = 0; x < size; x++)
  {
    for (y = 0; y < size; y++)
    {
      i[x][y] = ((x % 3) == 0) && ((y % 4) == 0);
    }
  }
}

/*
  Convention for writing cubes is:
    Bottom of cube: 0 1 2 3 where 0 is (0,0,0)
    Top of cube: 7 6 5 4 where 5 is (1,1,1)
  The order of operations is significant when Blender
  tries to read the .obj output file. Stay safe.
 */
void writeCube(FILE * fout, int x, int y, int z, int r)
{
  fprintf(fout, "v %d %d %d\n", x + 0, y + 0, z + 0);
  fprintf(fout, "v %d %d %d\n", x + r, y + 0, z + 0);
  fprintf(fout, "v %d %d %d\n", x + r, y + r, z + 0);
  fprintf(fout, "v %d %d %d\n", x + 0, y + r, z + 0);
  fprintf(fout, "v %d %d %d\n", x + r, y + r, z + r);
  fprintf(fout, "v %d %d %d\n", x + r, y + 0, z + r);
  fprintf(fout, "v %d %d %d\n", x + 0, y + 0, z + r);
  fprintf(fout, "v %d %d %d\n", x + 0, y + r, z + r);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 3, totalVerts + 2, totalVerts + 1, totalVerts + 0);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 7, totalVerts + 6, totalVerts + 5, totalVerts + 4);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 0, totalVerts + 1, totalVerts + 6, totalVerts + 7);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 3, totalVerts + 0, totalVerts + 7, totalVerts + 4);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 2, totalVerts + 3, totalVerts + 4, totalVerts + 5);
  fprintf(fout, "f %d// %d// %d// %d//\n", totalVerts + 1, totalVerts + 2, totalVerts + 5, totalVerts + 6);
  totalVerts++;
}

void writeVertex(FILE * fout, int x, int y, int z)
{
  fprintf(fout, "v %d %d %d\n", x, y, z);
}
